from typing import Dict

from airflow.sensors.base_sensor_operator import BaseSensorOperator
from airflow.utils.decorators import apply_defaults

from sai_airflow_plugins.operators.fabric_operator import FabricOperator


class FabricSensor(BaseSensorOperator, FabricOperator):
    """
    Executes a command on a remote host using the [Fabric](https://www.fabfile.org) library and returns True if and
    only if the exit code is 0. Like `FabricOperator` it uses a standard `SSHHook` for the connection configuration.

    The parameters for this sensor are the combined parameters of `FabricOperator` and `BaseSensorOperator`.
    """

    @apply_defaults
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def poke(self, context: Dict) -> bool:
        """
        Executes ``self.command`` over the configured SSH connection and checks its exit code.

        :param context: Context dict provided by airflow
        :return: True if the command's exit code was 0, else False.
        """
        result = self.execute_fabric_command()
        self.log.info(f"Fabric command exited with {result.exited}")

        return not result.exited



import getpass
import unittest

from airflow.exceptions import AirflowException
from faker import Faker
from invoke import FailingResponder
from paramiko.config import SSH_PORT

from sai_airflow_plugins.hooks.fabric_hook import FabricHook

faker = Faker()


class GetFabricConnectionTest(unittest.TestCase):

    def setUp(self):
        self.kwargs = dict(
            remote_host=faker.hostname(),
            username=faker.user_name(),
            password=faker.password(),
            key_file=faker.file_path(),
            port=faker.port_number(),
            timeout=faker.pyint(),
            inline_ssh_env=faker.pybool()
        )

        # These options can normally only be set when using an ssh_conn_id from db, not directly via the constructor
        self.pkey = faker.pystr()
        self.compress = faker.pybool()
        self.host_proxy = faker.hostname()

    def test_get_fabric_conn_all_opts(self):
        """
        The Connection object must have all options that were specified in FabricHook
        """
        hook = FabricHook(**self.kwargs)
        hook.pkey = self.pkey
        hook.compress = self.compress
        hook.host_proxy = self.host_proxy
        conn = hook.get_fabric_conn()

        self.assertEqual(conn.host, self.kwargs["remote_host"])
        self.assertEqual(conn.user, self.kwargs["username"])
        self.assertEqual(conn.port, self.kwargs["port"])
        self.assertEqual(conn.connect_timeout, self.kwargs["timeout"])
        self.assertEqual(conn.connect_kwargs["password"], self.kwargs["password"])
        self.assertEqual(conn.connect_kwargs["key_filename"], self.kwargs["key_file"])
        self.assertEqual(conn.connect_kwargs["pkey"], self.pkey)
        self.assertEqual(conn.connect_kwargs["compress"], self.compress)
        self.assertEqual(conn.connect_kwargs["sock"], self.host_proxy)
        self.assertEqual(conn.inline_ssh_env, self.kwargs["inline_ssh_env"])

    def test_get_fabric_conn_default_opts(self):
        """
        The Connection object must have default options that were specified in FabricHook or empty ones in case of
        optional connect_kwargs
        """
        hook = FabricHook(remote_host=self.kwargs["remote_host"])
        conn = hook.get_fabric_conn()

        self.assertEqual(conn.user, getpass.getuser())
        self.assertEqual(conn.port, SSH_PORT)
        self.assertEqual(conn.connect_timeout, 10)
        self.assertTrue(conn.connect_kwargs["compress"])
        self.assertNotIn("password", conn.connect_kwargs)
        self.assertNotIn("key_filename", conn.connect_kwargs)
        self.assertNotIn("pkey", conn.connect_kwargs)
        self.assertNotIn("sock", conn.connect_kwargs)
        self.assertFalse(conn.inline_ssh_env)


class RespondersTest(unittest.TestCase):

    def setUp(self):
        self.remote_host = faker.hostname()
        self.username = faker.user_name()
        self.password = faker.password()

    def test_sudo_pass_responder_requires_password(self):
        """
        Test that the sudo password responder requires a password in the hook object
        """
        hook = FabricHook(remote_host=self.remote_host)
        with self.assertRaises(AirflowException) as assertion:
            hook.get_sudo_pass_responder()
            self.assertIn("password", str(assertion.exception))

    def test_generic_pass_responder_requires_password(self):
        """
        Test that the generic password responder requires a password in the hook object
        """
        hook = FabricHook(remote_host=self.remote_host)
        with self.assertRaises(AirflowException) as assertion:
            hook.get_generic_pass_responder()
            self.assertIn("password", str(assertion.exception))

    def test_sudo_pass_responder_responds_with_password(self):
        """
        Test that the sudo password responder is a FailingResponder and responds with the password in the hook object
        """
        hook = FabricHook(remote_host=self.remote_host, username=self.username, password=self.password)
        responder = hook.get_sudo_pass_responder()
        self.assertIsInstance(responder, FailingResponder)
        self.assertEqual(responder.response, self.password + "\n")

    def test_generic_pass_responder_responds_with_password(self):
        """
        Test that the generic password responder is a FailingResponder responds with the password in the hook object
        """
        hook = FabricHook(remote_host=self.remote_host, username=self.username, password=self.password)
        responder = hook.get_generic_pass_responder()
        self.assertIsInstance(responder, FailingResponder)
        self.assertEqual(responder.response, self.password + "\n")



from typing import Optional, List, Dict, Any

from airflow.operators.http_operator import SimpleHttpOperator
from airflow.utils.decorators import apply_defaults

from sai_airflow_plugins.hooks.mattermost_webhook_hook import MattermostWebhookHook


class MattermostWebhookOperator(SimpleHttpOperator):
    """
    This operator allows you to post messages to Mattermost using incoming webhooks.
    It takes either a Mattermost webhook token directly or a connection that has a Mattermost webhook token.
    If both are supplied, http_conn_id will be used as base_url, and webhook_token will be taken as endpoint,
    the relative path of the url.

    Each Mattermost webhook token can be pre-configured to use a specific channel, username and icon. You can override
    these defaults in this operator.

    This operator is based on `airflow.contrib.operators.SlackWebhookOperator` as the Mattermost interface is largely
    similar to that of Slack.

    :param http_conn_id: connection that optionally has a Mattermost webhook token in the extra field
    :param webhook_token: Mattermost webhook token. If http_conn_id isn't supplied this should be the full webhook url.
    :param message: The message you want to send on Mattermost
    :param attachments: The attachments to send on Mattermost. Should be a list of dictionaries representing
                        Mattermost attachments.
    :param props: The props to send on Mattermost. Should be a dictionary representing Mattermost props.
    :param post_type: Sets an optional Mattermost post type, mainly for use by plugins. If supplied, must begin with
                      "custom_"
    :param channel: The channel the message should be posted to
    :param username: The username to post with
    :param icon_emoji: The emoji to use as icon for the user posting to Mattermost
    :param icon_url: The icon image URL string to use in place of the default icon.
    :param proxy: Proxy to use to make the Mattermost webhook call
    :param extra_options: Extra options for http hook
    """

    template_fields = ["webhook_token", "message", "attachments", "props", "post_type", "channel", "username",
                       "proxy", "extra_options"]

    @apply_defaults
    def __init__(self,
                 http_conn_id: Optional[str] = None,
                 webhook_token: Optional[str] = None,
                 message: str = "",
                 attachments: Optional[List[Dict[str, Any]]] = None,
                 props: Optional[Dict[str, Any]] = None,
                 post_type: Optional[str] = None,
                 channel: Optional[str] = None,
                 username: Optional[str] = None,
                 icon_emoji: Optional[str] = None,
                 icon_url: Optional[str] = None,
                 proxy: Optional[str] = None,
                 extra_options: Optional[Dict[str, Any]] = None,
                 *args,
                 **kwargs):
        super().__init__(endpoint=webhook_token, *args, **kwargs)
        self.http_conn_id = http_conn_id
        self.webhook_token = webhook_token
        self.message = message
        self.attachments = attachments
        self.props = props
        self.post_type = post_type
        self.channel = channel
        self.username = username
        self.icon_emoji = icon_emoji
        self.icon_url = icon_url
        self.proxy = proxy
        self.hook = None
        self.extra_options = extra_options

    def execute(self, context: Dict):
        """
        Call the MattermostWebhookHook to post the provided Mattermost message
        """
        self.hook = MattermostWebhookHook(
            self.http_conn_id,
            self.webhook_token,
            self.message,
            self.attachments,
            self.props,
            self.post_type,
            self.channel,
            self.username,
            self.icon_emoji,
            self.icon_url,
            self.proxy,
            self.extra_options
        )
        self.hook.execute()



import unittest
from unittest.mock import Mock

from airflow.exceptions import AirflowException
from faker import Faker
from invoke import Responder

from sai_airflow_plugins.operators.fabric_operator import FabricOperator
from tests.mocked_fabric_hook import MockedFabricHook

TEST_TASK_ID = "test_fabric_operator"

faker = Faker()


class FabricOperatorTest(unittest.TestCase):

    def setUp(self):
        self.hook = MockedFabricHook(
            remote_host=faker.hostname(),
            username=faker.user_name(),
            password=faker.password()
        )

    def test_ssh_conn_id_or_fabric_hook_required(self):
        """
        Test that either ssh_conn_id or fabric_hook is required for execution
        """
        op = FabricOperator(task_id=TEST_TASK_ID, command="ls")
        with self.assertRaises(AirflowException) as assertion:
            op.execute(context={})
            self.assertIn("ssh_conn_id", str(assertion.exception))
            self.assertIn("fabric_hook", str(assertion.exception))

    def test_command_required(self):
        """
        Test that command is required for execution
        """
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook)

        with self.assertRaises(AirflowException):
            op.execute(context={})

    def test_fabric_operator_execute_success(self):
        """
        Test that execution with a zero exit code succeeds and called connection.run with the correct parameters
        """
        command = faker.text()
        env = faker.pydict()
        pty = faker.pybool()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command=command, environment=env, get_pty=pty)

        self.assertTrue(op.execute(context={}))

        res = op.execute_fabric_command()
        res.conn.run.assert_called_with(dict(command=command, pty=pty, env=env, watchers=[], warn=True))

    def test_fabric_operator_use_sudo(self):
        """
        Test that parameter use_sudo calls connection.sudo with the correct parameters
        """
        command = faker.text()
        env = faker.pydict()
        pty = faker.pybool()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command=command, environment=env, get_pty=pty,
                            use_sudo=True)

        res = op.execute_fabric_command()
        res.conn.sudo.assert_called_with(dict(command=command, pty=pty, env=env, watchers=[], warn=True,
                                              password=self.hook.password))

    def test_fabric_operator_use_sudo_with_user(self):
        """
        Test that parameters use_sudo and sudo_user call connection.sudo with the correct parameters
        """
        command = faker.text()
        env = faker.pydict()
        pty = faker.pybool()
        sudo_user = faker.user_name()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command=command, environment=env, get_pty=pty,
                            use_sudo=True, sudo_user=sudo_user)

        res = op.execute_fabric_command()
        res.conn.sudo.assert_called_with(dict(command=command, pty=pty, env=env, watchers=[], warn=True,
                                              password=self.hook.password, user=sudo_user))

    def test_fabric_operator_execute_non_zero_exit(self):
        """
        Test that execution with a non-zero exit code fails and the exit code is logged
        """
        self.hook.exit_code = faker.pyint(min_value=1)
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls")

        with self.assertRaises(AirflowException) as assertion:
            op.execute(context={})
            self.assertIn(self.hook.exit_code, str(assertion.exception))

    def test_remote_host_override(self):
        """
        Test that remote_host can be overridden so it uses that one instead of hook.remote_host
        """
        remote_host = faker.hostname()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls", remote_host=remote_host)
        res = op.execute_fabric_command()

        self.assertEqual(res.conn.host, remote_host)

    def test_watcher(self):
        """
        Test that a watcher dict is converted correctly into the specified Watcher object
        """
        watcher_dict = {
            "class": Responder,
            "pattern": r"Ping\?:",
            "response": "Pong\n"
        }
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls", watchers=[watcher_dict])
        res = op.execute_fabric_command()
        watchers = res.conn.run.call_args[0][0]["watchers"]

        self.assertEqual(len(watchers), 1)
        self.assertIsInstance(watchers[0], Responder)
        self.assertEqual(watchers[0].pattern, watcher_dict["pattern"])
        self.assertEqual(watchers[0].response, watcher_dict["response"])

    def test_bad_watchers(self):
        """
        Test that a wrong watcher class or a watcher without the required parameters raises an exception
        """
        op1 = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls", watchers=[{"class": object}])
        with self.assertRaises(AirflowException):
            op1.execute_fabric_command()

        op2 = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls",
                             watchers=[{"class": Responder}])
        with self.assertRaises(AirflowException):
            op2.execute_fabric_command()

    def test_predefined_watchers(self):
        """
        Test that predefined watchers in `FabricHook` are correctly added when the add flags are True
        """
        self.hook.get_sudo_pass_responder = Mock()
        self.hook.get_generic_pass_responder = Mock()
        self.hook.get_unknown_host_key_responder = Mock()

        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls",
                            add_sudo_password_responder=True,
                            add_generic_password_responder=True,
                            add_unknown_host_key_responder=True)
        res = op.execute_fabric_command()
        watchers = res.conn.run.call_args[0][0]["watchers"]

        self.assertEqual(len(watchers), 3)
        self.hook.get_sudo_pass_responder.assert_called()
        self.hook.get_generic_pass_responder.assert_called()
        self.hook.get_unknown_host_key_responder.assert_called()

    def test_xcom_push(self):
        """
        Test that an XCom is pushed with the specified name as key and the stdout as value if `xcom_push_key` is set
        """
        task_inst = Mock()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls", xcom_push_key="test_xcom")
        op.execute(context={"task_instance": task_inst})

        task_inst.xcom_push.assert_called_with("test_xcom", self.hook.stdout)

    def test_strip_stdout(self):
        """
        Test that stdout is stripped if `strip_stdout` is True
        """
        command = faker.text()
        op = FabricOperator(task_id=TEST_TASK_ID, fabric_hook=self.hook, command=command, strip_stdout=True)

        res = op.execute_fabric_command()
        self.assertEqual(res.stdout, self.hook.stdout.strip())



import json
import unittest
from unittest.mock import patch

from airflow.exceptions import AirflowException
from faker import Faker

from sai_airflow_plugins.hooks.mattermost_webhook_hook import MattermostWebhookHook

faker = Faker()


class TestMattermostWebhookHook(unittest.TestCase):

    def test_http_conn_id_or_webhook_token_required(self):
        """
        Test that the hook constructor requires either a http_conn_id or a webhook_token
        """
        with self.assertRaises(AirflowException):
            MattermostWebhookHook()

        with patch(f"{__name__}.MattermostWebhookHook.get_connection"):
            MattermostWebhookHook(http_conn_id=faker.pystr())

        MattermostWebhookHook(webhook_token=faker.url())

    def test_execute(self):
        """
        Test that the hook execute method calls `HttpHook.run` with the correct endpoint, headers, options and
        json body
        """
        kwargs = dict(
            webhook_token=faker.url(),
            message=faker.text(),
            attachments=[faker.pydict(value_types=[str, int, bool]), faker.pydict(value_types=[str, int, bool])],
            props=faker.pydict(value_types=[str, int, bool]),
            post_type=faker.pystr(),
            channel=faker.pystr(),
            username=faker.user_name(),
            icon_emoji=faker.pystr(),
            icon_url=faker.url(),
            proxy=faker.url(),
            extra_options={"timeout": faker.pyint(), "allow_redirects": faker.pybool()}
        )
        hook = MattermostWebhookHook(**kwargs)

        expected_body = dict(
            text=kwargs["message"],
            attachments=kwargs["attachments"],
            props=kwargs["props"],
            type=kwargs["post_type"],
            channel=kwargs["channel"],
            username=kwargs["username"],
            icon_emoji=kwargs["icon_emoji"],
            icon_url=kwargs["icon_url"],
        )
        expected_extra_options = dict(
            **kwargs["extra_options"],
            proxies=dict(https=kwargs["proxy"])
        )

        with patch(f"{__name__}.MattermostWebhookHook.run") as mock_run:
            hook.execute()

            mock_run.assert_called_once()
            args = mock_run.call_args[1]
            self.assertEqual(args["endpoint"], kwargs["webhook_token"])
            self.assertEqual(args["headers"], {"Content-type": "application/json"})
            self.assertEqual(json.loads(args["data"]), expected_body)
            self.assertEqual(args["extra_options"], expected_extra_options)



from airflow.operators.bash_operator import BashOperator
from airflow.operators.dagrun_operator import TriggerDagRunOperator
from airflow.operators.python_operator import PythonOperator

from sai_airflow_plugins.operators.conditional_skip_mixin import ConditionalSkipMixin
from sai_airflow_plugins.operators.fabric_operator import FabricOperator


class ConditionalBashOperator(ConditionalSkipMixin, BashOperator):
    """
    Conditional bash operator.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~airflow.operators.bash_operator.BashOperator`
    """
    template_fields = BashOperator.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#ede4ff"


class ConditionalPythonOperator(ConditionalSkipMixin, PythonOperator):
    """
    Conditional python operator.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~airflow.operators.python_operator.PythonOperator`
    """
    template_fields = PythonOperator.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#ffebff"


class ConditionalTriggerDagRunOperator(ConditionalSkipMixin, TriggerDagRunOperator):
    """
    Conditional trigger DAG run operator.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~airflow.operators.dagrun_operator.TriggerDagRunOperator`
    """
    template_fields = TriggerDagRunOperator.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#efeaff"


class ConditionalFabricOperator(ConditionalSkipMixin, FabricOperator):
    """
    Conditional Fabric operator.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~sai_airflow_plugins.operators.fabric_operator.FabricOperator`
    """
    template_fields = FabricOperator.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#feffe5"



from airflow.contrib.hooks.ssh_hook import SSHHook
from airflow.exceptions import AirflowException
from fabric import Connection
from invoke import FailingResponder


class FabricHook(SSHHook):
    """
    This hook allows you to connect to an SSH remote host and run commands on it using the
    [Fabric](https://www.fabfile.org) library. It inherits from `SSHHook` and uses its input arguments for setting
    up the connection.

    :param ssh_conn_id: connection id from airflow Connections
    :param inline_ssh_env: whether to send environment variables "inline" as prefixes in front of command strings
                           (export VARNAME=value && mycommand here), instead of trying to submit them through the SSH
                           protocol itself (which is the default behavior). This is necessary if the remote server
                           has a restricted AcceptEnv setting (which is the common default).
    """

    def __init__(self,
                 ssh_conn_id: str = None,
                 inline_ssh_env: bool = False,
                 *args,
                 **kwargs):
        kwargs["ssh_conn_id"] = ssh_conn_id
        super().__init__(*args, **kwargs)
        self.inline_ssh_env = inline_ssh_env

    def get_fabric_conn(self) -> Connection:
        """
        Creates a Fabric `Connection` object using the settings in this hook.

        :return: `Connection` object
        """
        self.log.info(
            f"Setting up fabric connection for conn_id: {self.ssh_conn_id} to remote host: {self.remote_host}"
        )

        connect_kwargs = {
            "compress": self.compress
        }

        if self.password:
            password = self.password.strip()
            connect_kwargs["password"] = password

        if self.pkey:
            connect_kwargs["pkey"] = self.pkey

        if self.key_file:
            connect_kwargs["key_filename"] = self.key_file

        if self.host_proxy:
            connect_kwargs["sock"] = self.host_proxy

        return Connection(
            host=self.remote_host,
            user=self.username,
            port=self.port,
            connect_timeout=self.timeout,
            connect_kwargs=connect_kwargs,
            inline_ssh_env=self.inline_ssh_env
        )

    def get_sudo_pass_responder(self) -> FailingResponder:
        """
        Creates a responder for the sudo password prompt. It replies with the password of the SSH connection.
        Note: only use this if the SSH connection is configured with a password and not a key.

        :return: `FailingResponder` object; raises `AirflowException` if no password was configured in the connection
        """
        if not self.password:
            raise AirflowException("`add_sudo_password_responder` requires an SSH connection with a password.")

        return FailingResponder(
            pattern=r"\[sudo\] password for ",
            response=f"{self.password}\n",
            sentinel="Sorry, try again.\n"
        )

    def get_generic_pass_responder(self) -> FailingResponder:
        """
        Creates a responder for a generic password prompt. It replies with the password of the SSH connection. This is
        useful if you execute other ssh commands on the remote host, for example scp and rsync.
        Note: only use this if the SSH connection is configured with a password and not a key.

        :return: `FailingResponder` object; raises `AirflowException` if no password was configured in the connection
        """
        if not self.password:
            raise AirflowException("`add_generic_password_responder` requires an SSH connection with a password.")

        return FailingResponder(
            pattern=r"password: ",
            response=f"{self.password}\n",
            sentinel="Permission denied"
        )

    @staticmethod
    def get_unknown_host_key_responder() -> FailingResponder:
        """
        Creates a responder for a host authenticity check with an unknown key. It replies ``yes`` to continue
        connecting. This is useful if you execute other ssh commands on the remote host, for example scp and rsync.

        :return: `FailingResponder` object
        """
        return FailingResponder(
            pattern=r"Are you sure you want to continue connecting \(yes/no\)\? ",
            response="yes\n",
            sentinel="Host key verification failed.\n"
        )



from airflow.contrib.sensors.bash_sensor import BashSensor
from airflow.contrib.sensors.python_sensor import PythonSensor

from sai_airflow_plugins.operators.conditional_skip_mixin import ConditionalSkipMixin
from sai_airflow_plugins.sensors.fabric_sensor import FabricSensor


class ConditionalBashSensor(ConditionalSkipMixin, BashSensor):
    """
    Conditional bash sensor.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~airflow.contrib.sensors.bash_sensor.BashSensor`
    """
    template_fields = BashSensor.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#ede4ff"


class ConditionalPythonSensor(ConditionalSkipMixin, PythonSensor):
    """
    Conditional python sensor.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~airflow.contrib.sensors.bash_sensorPythonSensor`
    """
    template_fields = PythonSensor.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#ffebff"


class ConditionalFabricSensor(ConditionalSkipMixin, FabricSensor):
    """
    Conditional Fabric sensor.

    .. seealso:: :class:`~sai_airflow_plugins.operators.conditional_skip_mixin.ConditionalSkipMixin` and
                 :class:`~sai_airflow_plugins.sensors.fabric_sensor.FabricSensor`
    """
    template_fields = FabricSensor.template_fields + ConditionalSkipMixin.template_fields
    ui_color = "#e6f2eb"



from typing import Callable, Optional, Iterable, Dict

from airflow.exceptions import AirflowSkipException
from airflow.utils.decorators import apply_defaults


class ConditionalSkipMixin(object):
    """
    Mixin for making operators and sensors conditional. If the condition evaluates to True the operator or sensor
    executes normally, otherwise it skips the task.

    Note that you should correctly set the `template_field` in a derived class to include both the operator's
    and this mixin's templated fields. Example:

        class MyConditionalOperator(ConditionalSkipMixin, MyOperator):
            template_fields = MyOperator.template_fields + ConditionalSkipMixin.template_fields

    :param condition_callable: A callable that should evaluate to a truthy or falsy value to execute or skip the
                               task respectively. Note that Airflow's context is also passed as keyword arguments so
                               you need to define `**kwargs` in your function header. (templated)
    :param condition_kwargs: a dictionary of keyword arguments that will get unpacked in `condition_callable`.
                             (templated)
    :param condition_args: a list of positional arguments that will get unpacked in `condition_callable`. (templated)
    :param condition_provide_context: if set to true, Airflow will pass a set of keyword arguments that can be used in
                                      your condition callable. This set of kwargs correspond exactly to what you can
                                      use in your jinja templates. For this to work, you need to define `**kwargs` in
                                      your function header.
    """
    template_fields = ("condition_callable", "condition_args", "condition_kwargs")

    @apply_defaults
    def __init__(self,
                 condition_callable: Callable = False,
                 condition_args: Optional[Iterable] = None,
                 condition_kwargs: Optional[Dict] = None,
                 condition_provide_context: Optional[bool] = False,
                 *args,
                 **kwargs):
        super().__init__(*args, **kwargs)
        self.condition_callable = condition_callable
        self.condition_args = condition_args or []
        self.condition_kwargs = condition_kwargs or {}
        self.condition_provide_context = condition_provide_context
        self._condition_evaluated = False
        self._condition_value = None

    def execute(self, context: Dict):
        """
        If the condition evaluates to True execute the superclass `execute` method, otherwise skip the task.

        :param context: Context dict provided by airflow
        """
        if self._get_evaluated_condition_or_skip(context):
            super().execute(context)

    def poke(self, context: Dict) -> bool:
        """
        If the condition evaluates to True execute the superclass `poke` method, otherwise skip the task.

        :param context: Context dict provided by airflow
        :return: The result of the superclass `poke` method
        """
        if self._get_evaluated_condition_or_skip(context):
            return super().poke(context)

    def _get_evaluated_condition_or_skip(self, context: Dict):
        """
        Lazily evaluates `condition_callable` and raises `AirflowSkipException` if it's falsy.
        This is done because the `poke` method of a sensor may call the `execute` method as well.

        :param context: Context dict provided by airflow
        :return: The (cached) result of `condition_callable` if truthy, otherwise raises `AirflowSkipException`
        """
        if not self._condition_evaluated:

            if self.condition_provide_context:
                # Merge airflow's context and the callable's kwargs
                context.update(self.condition_kwargs)
                self.condition_kwargs = context

            self._condition_value = self.condition_callable(*self.condition_args, **self.condition_kwargs)

        if self._condition_value:
            return self._condition_value
        else:
            raise AirflowSkipException(
                f"Condition callable {self.condition_callable.__name__} evaluated to False. Skipping this task."
            )



from unittest.mock import Mock

from fabric import Connection

from sai_airflow_plugins.hooks.fabric_hook import FabricHook


class MockedFabricHook(FabricHook):
    exit_code = 0
    stdout = "Command completed successfully\n"

    def get_fabric_conn(self) -> Connection:
        """
        Mocks Fabric's connection.run and connection.sudo functions, returning a result with an exit code, stdout and
        the conn object
        """
        conn = super().get_fabric_conn()
        mock_result = Mock()
        mock_result.exited = self.exit_code
        mock_result.stdout = self.stdout
        mock_result.conn = conn
        conn.run = Mock(return_value=mock_result)
        conn.sudo = Mock(return_value=mock_result)
        return conn



import unittest
from unittest.mock import patch

from airflow.exceptions import AirflowSkipException
from airflow.models.baseoperator import BaseOperator
from airflow.sensors.base_sensor_operator import BaseSensorOperator
from faker import Faker

from sai_airflow_plugins.operators.conditional_skip_mixin import ConditionalSkipMixin

TEST_TASK_ID = "test_conditional_operator"

faker = Faker()


class ConditionalTestOperator(ConditionalSkipMixin, BaseOperator):
    pass


class ConditionalTestSensor(ConditionalSkipMixin, BaseSensorOperator):
    pass


class TestConditionalOperator(unittest.TestCase):

    def test_condition_true(self):
        """
        Test that the superclass execute is called when the callable evaluates to True
        """
        with patch(f"{__name__}.BaseOperator.execute") as mock_super_execute:
            op = ConditionalTestOperator(task_id=TEST_TASK_ID, condition_callable=lambda: True)
            op.execute(context={})
            mock_super_execute.assert_called_once()

    def test_condition_false(self):
        """
        Test that execute raises `AirflowSkipException` when the callable evaluates to False
        """
        with patch(f"{__name__}.BaseOperator.execute") as mock_super_execute:
            with self.assertRaises(AirflowSkipException):
                op = ConditionalTestOperator(task_id=TEST_TASK_ID, condition_callable=lambda: False)
                op.execute(context={})
                mock_super_execute.assert_not_called()

    def test_context_and_parameters(self):
        """
        Test that execute correctly supplies the callable with parameters and context if added
        """
        def test_callable(*args, **kwargs):
            self.assertEqual(args, (1,))
            self.assertEqual(kwargs, {"my_param": 2, "my_context_param": 3})
            return True

        with patch(f"{__name__}.BaseOperator.execute"):
            op = ConditionalTestOperator(task_id=TEST_TASK_ID,
                                         condition_callable=test_callable,
                                         condition_args=[1],
                                         condition_kwargs={"my_param": 2},
                                         condition_provide_context=True)
            op.execute({"my_context_param": 3})


class TestConditionalSensor(unittest.TestCase):

    def test_condition_true(self):
        """
        Test that the superclass poke is called when the callable evaluates to True
        """
        with patch(f"{__name__}.BaseSensorOperator.poke") as mock_super_poke:
            op = ConditionalTestSensor(task_id=TEST_TASK_ID, condition_callable=lambda: True)
            op.poke(context={})
            mock_super_poke.assert_called_once()

    def test_condition_false(self):
        """
        Test that poke raises `AirflowSkipException` when the callable evaluates to False
        """
        with patch(f"{__name__}.BaseSensorOperator.poke") as mock_super_poke:
            with self.assertRaises(AirflowSkipException):
                op = ConditionalTestSensor(task_id=TEST_TASK_ID, condition_callable=lambda: False)
                op.poke(context={})
                mock_super_poke.assert_not_called()

    def test_context_and_parameters(self):
        """
        Test that poke correctly supplies the callable with parameters and context if added
        """
        def test_callable(*args, **kwargs):
            self.assertEqual(args, (1,))
            self.assertEqual(kwargs, {"my_param": 2, "my_context_param": 3})
            return True

        with patch(f"{__name__}.BaseSensorOperator.poke"):
            op = ConditionalTestSensor(task_id=TEST_TASK_ID,
                                       condition_callable=test_callable,
                                       condition_args=[1],
                                       condition_kwargs={"my_param": 2},
                                       condition_provide_context=True)
            op.poke({"my_context_param": 3})



import json
from typing import Optional, List, Dict, Any

from airflow.exceptions import AirflowException
from airflow.hooks.http_hook import HttpHook


class MattermostWebhookHook(HttpHook):
    """
    This hook allows you to post messages to Mattermost using incoming webhooks.
    It takes either a Mattermost webhook token directly or a connection that has a Mattermost webhook token.
    If both are supplied, http_conn_id will be used as base_url, and webhook_token will be taken as endpoint,
    the relative path of the url.

    Each Mattermost webhook token can be pre-configured to use a specific channel, username and icon. You can override
    these defaults in this hook.

    This hook is based on `airflow.contrib.hooks.SlackWebhookHook` as the Mattermost interface is largely similar
    to that of Slack.

    :param http_conn_id: connection that optionally has a Mattermost webhook token in the extra field
    :param webhook_token: Mattermost webhook token. If http_conn_id isn't supplied this should be the full webhook url.
    :param message: The message you want to send on Mattermost
    :param attachments: The attachments to send on Mattermost. Should be a list of dictionaries representing
                        Mattermost attachments.
    :param props: The props to send on Mattermost. Should be a dictionary representing Mattermost props.
    :param post_type: Sets an optional Mattermost post type, mainly for use by plugins. If supplied, must begin with
                      "custom_"
    :param channel: The channel the message should be posted to
    :param username: The username to post with
    :param icon_emoji: The emoji to use as icon for the user posting to Mattermost
    :param icon_url: The icon image URL string to use in place of the default icon.
    :param proxy: Proxy to use to make the Mattermost webhook call
    :param extra_options: Extra options for http hook
    """

    def __init__(self,
                 http_conn_id: Optional[str] = None,
                 webhook_token: Optional[str] = None,
                 message: str = "",
                 attachments: Optional[List[Dict[str, Any]]] = None,
                 props: Optional[Dict[str, Any]] = None,
                 post_type: Optional[str] = None,
                 channel: Optional[str] = None,
                 username: Optional[str] = None,
                 icon_emoji: Optional[str] = None,
                 icon_url: Optional[str] = None,
                 proxy: Optional[str] = None,
                 extra_options: Optional[Dict[str, Any]] = None,
                 *args,
                 **kwargs):
        super().__init__(http_conn_id=http_conn_id, *args, **kwargs)
        self.webhook_token = self._get_token(webhook_token, http_conn_id)
        self.message = message
        self.attachments = attachments
        self.props = props
        self.post_type = post_type
        self.channel = channel
        self.username = username
        self.icon_emoji = icon_emoji
        self.icon_url = icon_url
        self.proxy = proxy
        self.extra_options = extra_options or {}

    def _get_token(self, token: str, http_conn_id: str) -> str:
        """
        Given either a manually set token or a conn_id, return the webhook_token to use.

        :param token: The manually provided token
        :param http_conn_id: The conn_id provided
        :return: webhook_token to use
        """
        if token:
            return token
        elif http_conn_id:
            conn = self.get_connection(http_conn_id)
            extra = conn.extra_dejson
            return extra.get("webhook_token", "")
        else:
            raise AirflowException("Cannot get webhook token: no valid Mattermost webhook token nor conn_id supplied")

    def _build_mattermost_message(self) -> str:
        """
        Construct the Mattermost message. All relevant parameters are combined here to a valid Mattermost json body.

        :return: Mattermost JSON body to send
        """
        cmd = {}

        if self.channel:
            cmd["channel"] = self.channel
        if self.username:
            cmd["username"] = self.username
        if self.icon_emoji:
            cmd["icon_emoji"] = self.icon_emoji
        if self.icon_url:
            cmd["icon_url"] = self.icon_url
        if self.attachments:
            cmd["attachments"] = self.attachments
        if self.props:
            cmd["props"] = self.props
        if self.post_type:
            cmd["type"] = self.post_type

        cmd["text"] = self.message
        return json.dumps(cmd)

    def execute(self):
        """
        Execute the Mattermost webhook call
        """

        if self.proxy:
            # we only need https proxy for Mattermost, as the endpoint is https
            self.extra_options.update({"proxies": {"https": self.proxy}})

        mattermost_message = self._build_mattermost_message()
        self.run(endpoint=self.webhook_token,
                 data=mattermost_message,
                 headers={"Content-type": "application/json"},
                 extra_options=self.extra_options)



import unittest
from unittest.mock import patch

from faker import Faker

from sai_airflow_plugins.operators.mattermost_webhook_operator import MattermostWebhookOperator

TEST_TASK_ID = "test_mattermost_webhook_operator"

faker = Faker()


class TestMattermostWebhookOperator(unittest.TestCase):

    def test_execute(self):
        """
        Test that execute creates a `MattermostWebhookHook` with all parameters and calls its execute method
        """
        kwargs = dict(
            webhook_token=faker.url(),
            message=faker.text(),
            attachments=[faker.pydict(value_types=[str, int, bool]), faker.pydict(value_types=[str, int, bool])],
            props=faker.pydict(value_types=[str, int, bool]),
            post_type=faker.pystr(),
            channel=faker.pystr(),
            username=faker.user_name(),
            icon_emoji=faker.pystr(),
            icon_url=faker.url(),
            proxy=faker.url(),
            extra_options={"timeout": faker.pyint(), "allow_redirects": faker.pybool()}
        )
        op = MattermostWebhookOperator(task_id=TEST_TASK_ID, **kwargs)

        with patch("sai_airflow_plugins.operators.mattermost_webhook_operator.MattermostWebhookHook.execute"
                   ) as mock_hook_execute:
            op.execute(context={})
            mock_hook_execute.assert_called_once()

            for arg_name, arg_val in kwargs.items():
                self.assertEqual(getattr(op.hook, arg_name), arg_val)



from setuptools import setup, find_packages

# Read long description from README.rst
with open("README.rst", "r") as f:
    long_description = f.read()

# Read version number from VERSION
with open("VERSION", "r") as f:
    version = f.read()

requirements = [
    "apache-airflow[ssh,http]>=1.10",
    "fabric>=2.5"
]

requirements_docs = [
    "sphinx>=3.5,<4",
    "sphinx-rtd-theme",
    "sphinx-autodoc-typehints",
    "sphinx-versions==1.0.1"
]

requirements_tests = [
    "pytest",
    "faker"
]

setup(
    name="sai-airflow-plugins",
    version=version,
    author="Slimmer.AI",
    description="A Python package with various operators, hooks and utilities for Apache Airflow",
    long_description=long_description,
    url="https://github.com/Slimmer-AI/sai-airflow-plugins",
    license="Apache License 2.0",
    license_files=["LICENSE"],
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    extras_require={"docs": requirements_docs,
                    'tests': requirements_tests},
    test_suite="tests",
    tests_require=requirements_tests,
    classifiers=[
        # Development status
        "Development Status :: 5 - Production/Stable",

        # Environments
        "Environment :: Plugins",

        # Supported Python versions
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",

        # License
        "License :: OSI Approved :: Apache Software License",

        # Topic
        "Topic :: System :: Monitoring"
    ]
)



from typing import Dict, List, Any, Optional

from airflow.exceptions import AirflowException
from airflow.models.baseoperator import BaseOperator
from airflow.utils.decorators import apply_defaults
from fabric import Result
from invoke import Responder, StreamWatcher

from sai_airflow_plugins.hooks.fabric_hook import FabricHook


class FabricOperator(BaseOperator):
    """
    Operator to execute commands on a remote host using the [Fabric](https://www.fabfile.org) library.
    It uses `FabricHook` for the connection configuration, which is derived from the standard `SSHHook`.

    The advantage of this operator over the standard `SSHOperator` is that you can add watchers that respond to
    specific command output. A number of predefined watchers are included in this operator. Note, however, that
    some of these require the FabricHook to be configured with a password and not a private key.

    :param fabric_hook: predefined fabric_hook to use for remote execution. Either `fabric_hook` or `ssh_conn_id` needs
                        to be provided.
    :param ssh_conn_id: connection id from airflow Connections. `ssh_conn_id` will be ignored if `fabric_hook` is
                        provided. (templated)
    :param remote_host: remote host to connect. (templated) Nullable. If provided, it will replace the `remote_host`
                        which was defined in `fabric_hook` or predefined in the connection of `ssh_conn_id`.
    :param command: command to execute on remote host. (templated)
    :param use_sudo: uses Fabric's sudo function instead of run. Because this function automatically adds a responder
                     for the password prompt, parameter `add_sudo_password_responder` will be ignored. It uses the
                     SSH connection's password as reply.
    :param sudo_user: If `use_sudo` is `True`, run the command as this user. If not supplied it will run as root.
                      This parameter is ignored if `use_sudo` is `False`.
    :param watchers: Watchers for responding to specific command output. This is a list of dicts specifying a class
                     of type ``StreamWatcher`` and its arguments. For each dict the corresponding object is created
                     and added to Fabric's run function. It's done this way because arguments to an operator are
                     pickled and StreamWatcher objects are derived from thread.local which can't be pickled.
                     Example:
                     >>> {"watchers": [{"class": Responder, "pattern": r"Continue\\?", "response": "yes\\n"}]}
                     See also: http://docs.pyinvoke.org/en/latest/concepts/watchers.html
    :param add_sudo_password_responder: adds a responder for the sudo password prompt. It replies with the password of
                                        the SSH connection. Set this to True if your command contains one or more sudo
                                        statements. If you use `use_sudo` instead, then don't use this parameter.
    :param add_generic_password_responder: adds a responder for a generic password prompt. It replies with the password
                                           of the SSH connection. This is useful if you execute other ssh commands on
                                           the remote host, for example scp and rsync.
    :param add_unknown_host_key_responder: adds a responder for a host authenticity check with an unknown key.
                                           It replies ``yes`` to continue connecting. This is useful if you execute
                                           other ssh commands on the remote host, for example scp and rsync.
    :param connect_timeout: Connection timeout, in seconds. The default is 10.
    :param environment: a dict of shell environment variables. Note that the server will reject them silently if
                        `AcceptEnv` is not set in SSH config. In such cases setting `inline_ssh_env` to True may help.
                        (templated)
    :param inline_ssh_env: whether to send environment variables "inline" as prefixes in front of command strings
                           (export VARNAME=value && mycommand here), instead of trying to submit them through the SSH
                           protocol itself (which is the default behavior). This is necessary if the remote server
                           has a restricted AcceptEnv setting (which is the common default).
    :param xcom_push_key: push stdout to an XCom with this key. If None (default), or stdout is empty, then no XCom
                          will be pushed.
    :param strip_stdout: strip leading and trailing whitespace from stdout. Useful, for example, when pushing stdout
                         to an XCom and you don't want a trailing newline in it.
    :param get_pty: request a pseudo-terminal from the server, instead of connecting directly to the stdout/stderr
                    streams. This may be necessary when running programs that require a terminal. Note that stderr
                    output will be included in stdout, and thus added to an XCom when using `xcom_push_key`.
    """

    template_fields = ("ssh_conn_id", "command", "remote_host", "environment")
    template_ext = (".sh",)
    ui_color = "#ebfaff"

    @apply_defaults
    def __init__(self,
                 fabric_hook: Optional[FabricHook] = None,
                 ssh_conn_id: Optional[str] = None,
                 remote_host: Optional[str] = None,
                 command: str = None,
                 use_sudo: Optional[bool] = False,
                 sudo_user: Optional[str] = None,
                 watchers: Optional[List[Dict[str, Any]]] = None,
                 add_sudo_password_responder: Optional[bool] = False,
                 add_generic_password_responder: Optional[bool] = False,
                 add_unknown_host_key_responder: Optional[bool] = False,
                 connect_timeout: Optional[int] = 10,
                 environment: Optional[Dict[str, Any]] = None,
                 inline_ssh_env: Optional[bool] = False,
                 xcom_push_key: Optional[str] = None,
                 strip_stdout: Optional[bool] = False,
                 get_pty: Optional[bool] = False,
                 *args,
                 **kwargs):
        super().__init__(*args, **kwargs)
        self.fabric_hook = fabric_hook
        self.ssh_conn_id = ssh_conn_id
        self.remote_host = remote_host
        self.command = command
        self.use_sudo = use_sudo
        self.sudo_user = sudo_user
        self.watchers = watchers or []
        self.add_sudo_password_responder = False if self.use_sudo else add_sudo_password_responder
        self.add_generic_password_responder = add_generic_password_responder
        self.add_unknown_host_key_responder = add_unknown_host_key_responder
        self.connect_timeout = connect_timeout
        self.environment = environment or {}
        self.inline_ssh_env = inline_ssh_env
        self.xcom_push_key = xcom_push_key
        self.strip_stdout = strip_stdout
        self.get_pty = get_pty

    def execute(self, context: Dict):
        """
        Executes ``self.command`` over the configured SSH connection.

        :param context: Context dict provided by airflow
        :return: True or the connection's stdout if ``self.do_xcom_push`` is True.
                 On an error, raises AirflowException.
        """
        result = self.execute_fabric_command()

        if result.exited == 0:
            # Push the output to an XCom if requested
            if self.xcom_push_key and result.stdout:
                task_inst = context["task_instance"]
                task_inst.xcom_push(self.xcom_push_key, result.stdout)

            return True
        else:
            raise AirflowException(f"Command exited with return code {result.exited}. See log output for details.")

    def execute_fabric_command(self) -> Result:
        """
        Executes ``self.command`` over the configured SSH connection.

        :return: The `Result` object from Fabric's `run` method
        """
        try:
            if self.fabric_hook and isinstance(self.fabric_hook, FabricHook):
                if self.ssh_conn_id:
                    self.log.info("ssh_conn_id is ignored when fabric_hook is provided.")

                if self.remote_host is not None:
                    self.log.info("remote_host is provided explicitly. It will replace the remote_host which was "
                                  "defined in fabric_hook.")
                    self.fabric_hook.remote_host = self.remote_host

            elif self.ssh_conn_id:
                self.log.info("fabric_hook is not provided or invalid. Trying ssh_conn_id to create FabricHook.")
                if self.remote_host is None:
                    self.fabric_hook = FabricHook(ssh_conn_id=self.ssh_conn_id,
                                                  timeout=self.connect_timeout,
                                                  inline_ssh_env=self.inline_ssh_env)
                else:
                    # Prevent empty `SSHHook.remote_host` field which would otherwise raise an exception
                    self.log.info("remote_host is provided explicitly. It will replace the remote_host which was "
                                  "predefined in the connection specified by ssh_conn_id.")
                    self.fabric_hook = FabricHook(ssh_conn_id=self.ssh_conn_id,
                                                  remote_host=self.remote_host,
                                                  timeout=self.connect_timeout,
                                                  inline_ssh_env=self.inline_ssh_env)

            else:
                raise AirflowException("Cannot operate without fabric_hook or ssh_conn_id.")

            if not self.command:
                raise AirflowException("SSH command not specified. Aborting.")

            conn = self.fabric_hook.get_fabric_conn()

            # Create watcher objects, using the provided dictionary to instantiate the class and supply its kwargs
            watchers = []
            for watcher_dict in self.watchers:
                try:
                    watcher_class = watcher_dict.pop("class")
                except KeyError:
                    self.log.info(f"Watcher class missing. Defaulting to {Responder}.")
                    watcher_class = Responder

                if not issubclass(watcher_class, StreamWatcher):
                    raise AirflowException(
                        f"The class attribute of a watcher dict must contain a subclass of {StreamWatcher}."
                    )

                watcher = watcher_class(**watcher_dict)
                watchers.append(watcher)

            # Add predefined watchers
            if self.add_sudo_password_responder:
                watchers.append(self.fabric_hook.get_sudo_pass_responder())

            if self.add_generic_password_responder:
                watchers.append(self.fabric_hook.get_generic_pass_responder())

            if self.add_unknown_host_key_responder:
                watchers.append(self.fabric_hook.get_unknown_host_key_responder())

            if self.use_sudo:
                if self.sudo_user:
                    self.log.info(f"Running sudo command as '{self.sudo_user}': {self.command}")
                else:
                    self.log.info(f"Running sudo command: {self.command}")
            else:
                self.log.info(f"Running command: {self.command}")

            if self.environment:
                formatted_env_msg = "\n".join(f"{k}={v}" for k, v in self.environment.items())
                self.log.info(f"With environment variables:\n{formatted_env_msg}")

            run_kwargs = dict(
                command=self.command,
                pty=self.get_pty,
                env=self.environment,
                watchers=watchers,
                warn=True  # don't directly raise an UnexpectedExit when the exit code is non-zero
            )

            if self.use_sudo:
                run_kwargs["password"] = self.fabric_hook.password
                if self.sudo_user:
                    run_kwargs["user"] = self.sudo_user
                res = conn.sudo(run_kwargs)
            else:
                res = conn.run(run_kwargs)

            # Strip stdout if requested
            if res.stdout and self.strip_stdout:
                res.stdout = res.stdout.strip()

            return res

        except Exception as e:
            raise AirflowException(f"Fabric operator error: {e}")



import unittest

from faker import Faker

from sai_airflow_plugins.sensors.fabric_sensor import FabricSensor
from tests.mocked_fabric_hook import MockedFabricHook

TEST_TASK_ID = "test_fabric_sensor"

faker = Faker()


class FabricSensorTest(unittest.TestCase):

    def setUp(self):
        self.hook = MockedFabricHook(
            remote_host=faker.hostname(),
            username=faker.user_name(),
            password=faker.password()
        )

    def test_fabric_sensor_with_zero_exit_code(self):
        """
        Test that poke returns True in case of a zero exit code
        """
        op = FabricSensor(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls")
        self.assertTrue(op.poke(context={}))

    def test_fabric_sensor_with_non_zero_exit_code(self):
        """
        Test that poke returns True in case of a zero exit code
        """
        self.hook.exit_code = faker.pyint(min_value=1)
        op = FabricSensor(task_id=TEST_TASK_ID, fabric_hook=self.hook, command="ls")
        self.assertFalse(op.poke(context={}))
